@@ -23,6 +23,7 @@
 
 #include "mc-array-private.h"
 #include "mc-fle2-range-operator-private.h"
+#include "mc-optional-private.h"
 
 /** FLE2FindRangePayloadEdgesInfoV2 represents the token information for a range
  * find query. It is encoded inside an FLE2FindRangePayloadV2.
@@ -44,8 +45,17 @@ typedef struct {
  * } FLE2FindRangePayloadV2;
  *
  * bson is a BSON document of this form:
- * g: array<EdgeFindTokenSetV2> // Array of Edges
- * cm: <int64> // Queryable Encryption max counter
+ * payload: <document>
+ *  g: array<EdgeFindTokenSetV2> // Array of Edges
+ *  cm: <int64> // Queryable Encryption max counter
+ * payloadId: <int32> // Payload ID.
+ * firstOperator: <int32>
+ * secondOperator: <int32>
+ * sp: optional<int64> // Sparsity.
+ * pn: optional<int32> // Precision.
+ * tf: optional<int32> // Trim Factor.
+ * mn: optional<any> // Index Min.
+ * mx: optional<any> // Index Max.
  */
 typedef struct {
     struct {
@@ -61,6 +71,11 @@ typedef struct {
     // secondOperator represents the second query operator for which this payload
     // was generated. Only populated for two-sided ranges. It is 0 if unset.
     mc_FLE2RangeOperator_t secondOperator;
+    mc_optional_int64_t sparsity;   // sp
+    mc_optional_int32_t precision;  // pn
+    mc_optional_int32_t trimFactor; // tf
+    bson_value_t indexMin;          // mn
+    bson_value_t indexMax;          // mx
 } mc_FLE2FindRangePayloadV2_t;
 
 /**
@@ -81,7 +96,7 @@ typedef struct {
 
 void mc_FLE2FindRangePayloadV2_init(mc_FLE2FindRangePayloadV2_t *payload);
 
-bool mc_FLE2FindRangePayloadV2_serialize(const mc_FLE2FindRangePayloadV2_t *payload, bson_t *out);
+bool mc_FLE2FindRangePayloadV2_serialize(const mc_FLE2FindRangePayloadV2_t *payload, bson_t *out, bool use_range_v2);
 
 void mc_FLE2FindRangePayloadV2_cleanup(mc_FLE2FindRangePayloadV2_t *payload);
 
