@@ -16,6 +16,7 @@
 
 #include <mongocrypt.h>
 
+#include "kms_message/kms_b64.h"
 #include "mongocrypt-crypto-private.h"
 #include "mongocrypt-private.h"
 #include "test-mongocrypt.h"
@@ -94,7 +95,7 @@ static void _test_create_data_key_with_provider(_mongocrypt_tester_t *tester,
         BSON_ASSERT(mongocrypt_ctx_state(ctx) == MONGOCRYPT_CTX_NEED_KMS);
         kms = mongocrypt_ctx_next_kms_ctx(ctx);
         BSON_ASSERT(kms);
-        ASSERT_OK(mongocrypt_kms_ctx_feed(kms, TEST_FILE("./test/data/kms-encrypt-reply.txt")), kms);
+        ASSERT_OK(mongocrypt_kms_ctx_feed(kms, TEST_FILE("./test/data/kms-aws/encrypt-response.txt")), kms);
         BSON_ASSERT(0 == mongocrypt_kms_ctx_bytes_needed(kms));
         ASSERT_OK(mongocrypt_ctx_kms_done(ctx), ctx);
     }
@@ -180,7 +181,7 @@ static void _test_datakey_custom_endpoint(_mongocrypt_tester_t *tester) {
     bin = mongocrypt_binary_new();
     ASSERT_OK(mongocrypt_kms_ctx_message(kms_ctx, bin), ctx);
     BSON_ASSERT(NULL != strstr((char *)bin->data, "Host:example.com"));
-    ASSERT_OK(mongocrypt_kms_ctx_feed(kms_ctx, TEST_FILE("./test/data/kms-encrypt-reply.txt")), kms_ctx);
+    ASSERT_OK(mongocrypt_kms_ctx_feed(kms_ctx, TEST_FILE("./test/data/kms-aws/encrypt-response.txt")), kms_ctx);
     BSON_ASSERT(0 == mongocrypt_kms_ctx_bytes_needed(kms_ctx));
     ASSERT_OK(mongocrypt_ctx_kms_done(ctx), ctx);
 
@@ -210,7 +211,7 @@ static void _test_datakey_kms_per_ctx_credentials(_mongocrypt_tester_t *tester)
     crypt = mongocrypt_new();
     mongocrypt_setopt_use_need_kms_credentials_state(crypt);
     ASSERT_OK(mongocrypt_setopt_kms_providers(crypt, TEST_BSON("{'aws': {}}")), crypt);
-    ASSERT_OK(mongocrypt_init(crypt), crypt);
+    ASSERT_OK(_mongocrypt_init_for_test(crypt), crypt);
     ctx = mongocrypt_ctx_new(crypt);
     ASSERT_OK(mongocrypt_ctx_setopt_masterkey_aws(ctx, "region", -1, "cmk", -1), ctx);
     ASSERT_OK(mongocrypt_ctx_setopt_masterkey_aws_endpoint(ctx, "example.com", -1), ctx);
@@ -228,7 +229,7 @@ static void _test_datakey_kms_per_ctx_credentials(_mongocrypt_tester_t *tester)
     bin = mongocrypt_binary_new();
     ASSERT_OK(mongocrypt_kms_ctx_message(kms_ctx, bin), ctx);
     BSON_ASSERT(NULL != strstr((char *)bin->data, "Host:example.com"));
-    ASSERT_OK(mongocrypt_kms_ctx_feed(kms_ctx, TEST_FILE("./test/data/kms-encrypt-reply.txt")), kms_ctx);
+    ASSERT_OK(mongocrypt_kms_ctx_feed(kms_ctx, TEST_FILE("./test/data/kms-aws/encrypt-response.txt")), kms_ctx);
     BSON_ASSERT(0 == mongocrypt_kms_ctx_bytes_needed(kms_ctx));
     ASSERT_OK(mongocrypt_ctx_kms_done(ctx), ctx);
 
@@ -258,7 +259,7 @@ static void _test_datakey_kms_per_ctx_credentials_not_requested(_mongocrypt_test
                                               TEST_BSON("{'aws': {}, 'azure': {'tenantId': '', 'clientId': "
                                                         "'', 'clientSecret': '' }}")),
               crypt);
-    ASSERT_OK(mongocrypt_init(crypt), crypt);
+    ASSERT_OK(_mongocrypt_init_for_test(crypt), crypt);
     ctx = mongocrypt_ctx_new(crypt);
     ASSERT_OK(mongocrypt_ctx_setopt_key_encryption_key(ctx,
                                                        TEST_BSON("{'provider': 'azure', 'keyVaultEndpoint': "
@@ -280,13 +281,13 @@ static void _test_datakey_kms_per_ctx_credentials_local(_mongocrypt_tester_t *te
     mongocrypt_binary_t *bin;
     bson_t key_bson;
     bson_iter_t iter;
-    const char *local_kek = "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
-                            "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA";
+    uint8_t local_kek_raw[MONGOCRYPT_KEY_LEN] = {0};
+    char *local_kek = kms_message_raw_to_b64(local_kek_raw, sizeof(local_kek_raw));
 
     crypt = mongocrypt_new();
     mongocrypt_setopt_use_need_kms_credentials_state(crypt);
     ASSERT_OK(mongocrypt_setopt_kms_providers(crypt, TEST_BSON("{'local': {}}")), crypt);
-    ASSERT_OK(mongocrypt_init(crypt), crypt);
+    ASSERT_OK(_mongocrypt_init_for_test(crypt), crypt);
     ctx = mongocrypt_ctx_new(crypt);
     ASSERT_OK(mongocrypt_ctx_setopt_key_encryption_key(ctx, TEST_BSON("{'provider': 'local' }")), ctx);
     ASSERT_OK(mongocrypt_ctx_datakey_init(ctx), ctx);
@@ -310,6 +311,7 @@ static void _test_datakey_kms_per_ctx_credentials_local(_mongocrypt_tester_t *te
     mongocrypt_binary_destroy(bin);
     mongocrypt_ctx_destroy(ctx);
     mongocrypt_destroy(crypt);
+    bson_free(local_kek);
 }
 
 static void _test_datakey_custom_key_material(_mongocrypt_tester_t *tester) {
@@ -377,8 +379,11 @@ static void _test_datakey_custom_key_material(_mongocrypt_tester_t *tester) {
         _mongocrypt_buffer_t decrypted_dek_buf;
         mongocrypt_binary_t decrypted_dek;
 
+        mc_kms_creds_t kc;
+        ASSERT(_mongocrypt_opts_kms_providers_lookup(&crypt->opts.kms_providers, "local", &kc));
+
         ASSERT(_mongocrypt_unwrap_key(crypt->crypto,
-                                      &crypt->opts.kms_providers.local.key,
+                                      &kc.value.local.key,
                                       &encrypted_dek_buf,
                                       &decrypted_dek_buf,
                                       crypt->status));
